'''犯人'''
from ..cat import cat
from ..model import Game
from ..config import R


@cat.on_cmd(cmds=R.犯人, states="game", auto_help=False)
async def criminal():
    # 排除私聊发送的消息
    if cat.event.origin_channel:
        return

    game = cat.get_data(Game)

    # 排除未参加游戏的人
    player = game.get_player(cat.user.id)
    if not player:
        return

    async with game.lock:
        card = cat.cmd
        if not await player.check(card, max_num=1):
            return

        await player.play_card(card)
        player.is_good = False
        await game.send(f"{R.犯人}是 {player.index_name}！")

        # 被警部抓到了
        if game.police.target_id == player.id:
            p2 = game.get_player(game.police.owner_id)
            p2.is_good = True
            await game.send(f"但是{R.犯人}被 {p2.index_name} 布置的{R.警部}抓到了！")
            return await game.end(True)

        await game.end(False)



'''神犬'''
from .overtime import set_overtime_task
from ...cat import cat
from ...model import Game
from ...config import R


@cat.on_cmd(cmds=R.神犬, states="game", auto_help=False)
async def dog():
    # 排除私聊发送的消息
    if cat.event.origin_channel:
        return

    game = cat.get_data(Game)

    # 排除未参加游戏的人
    player = game.get_player(cat.user.id)
    if not player:
        return

    async with game.lock:
        card = cat.cmd
        if not await player.check(card, at_require=True):
            return

        if cat.event.at == player.id:
            return await game.send(f"{R.神犬}不咬主人")

        p2 = game.get_player(cat.event.at)
        if not p2.cards:
            return await game.send(f"{R.神犬}只能扑向有手牌的玩家")

        game.dog.target_id = p2.id
        game.dog.owner_id = cat.user.id

        await player.play_card(card)
        await game.send(f"目标是 {p2.index_name}！请TA丢弃一张牌")

        game.set_state("dog")

        # 设置超时任务
        set_overtime_task(p2)


@cat.on_cmd(cmds=[R.共犯, R.普通人, R.不在场证明, R.目击者, R.侦探, R.交易, R.谣言, R.情报交换, R.警部, R.犯人], states="dog", auto_help=False)
async def dog_bite():
    # 排除私聊发送的消息
    if cat.event.origin_channel:
        return

    game = cat.get_data(Game)

    # 排除未参加游戏的人
    player = game.get_player(cat.user.id)
    if not player:
        return

    async with game.lock:
        card = cat.cmd

        if game.dog.target_id != player.id:
            return

        if card not in player.cards:
            return await game.send(f"{player.index_name} 没有{card}")

        await player.play_card(card)

        if card == R.犯人:
            player.is_good = False
            owner = game.get_player(game.dog.owner_id)
            owner.is_good = True
            return await game.end(True)

        player.cards.append(R.神犬)
        await game.send(f"{player.index_name} 获得{R.神犬}牌")

        game.set_state("game")
        await game.turn_next()



from random import sample
from ..config import R


# 独特池
unique_3 = [R.第一发现人, R.犯人, R.侦探, R.不在场证明]
unique_4 = [R.第一发现人, R.犯人, R.侦探, R.不在场证明, R.共犯]
unique_5 = [R.第一发现人, R.犯人, R.侦探, R.不在场证明, R.不在场证明, R.共犯]
unique_6 = [R.第一发现人, R.犯人, R.侦探, R.侦探, R.不在场证明, R.不在场证明, R.共犯, R.共犯]
unique_7 = [R.第一发现人, R.犯人, R.侦探, R.侦探, R.不在场证明, R.不在场证明, R.不在场证明, R.共犯, R.共犯]
unique_8 = [
    R.第一发现人, R.犯人, R.侦探, R.侦探, R.侦探, R.不在场证明, R.不在场证明, R.不在场证明, R.不在场证明, R.不在场证明, R.共犯, R.共犯
]
unique = [
    0, 0, 0,
    unique_3,
    unique_4,
    unique_5,
    unique_6,
    unique_7,
    unique_8,
]

# 普通池
normal = [
    R.普通人, R.普通人, R.谣言, R.谣言, R.谣言, R.谣言, R.情报交换, R.情报交换, R.情报交换, R.情报交换, R.目击者, R.目击者, R.目击者, R.目击者, R.交易, R.交易, R.交易, R.交易
]
normal_n = [
    0, 0, 0,
    8,
    11,
    14,
    15,
    17,
    18,
]

# 加强池
pro = [R.神犬, R.警部]
pro_n = [
    0, 0, 0,
    0,
    0,
    0,
    1,
    2,
    2,
]


def get_cards(n: int):
    return unique[n] + sample(normal, normal_n[n]) + sample(pro, pro_n[n])



'''情报交换超时'''
import asyncio
from random import choice
from ...model import Player
from ...config import config


def set_overtime_task(player: Player):
    '''设置超时任务'''
    loop = asyncio.get_event_loop()
    player.fut = loop.create_future()
    loop.create_task(overtime(player))


async def overtime(player: Player):
    try:
        await asyncio.wait_for(player.fut, config.overtime)
    except asyncio.exceptions.TimeoutError:
        game = player.game
        give = game.round_give.get_give(player.id)
        async with game.lock:
            card = choice(player.cards)
            give.card = card
            await player.send(f"您被系统强制交出了{card}")
            await game.send(f"{give.giver.index_name} 已决定好卡牌")

            # 判断是否完成
            if game.round_give.all_given:
                # 私聊 互相给牌
                game.round_give.set_receivers()
                await game.round_give.convey_all()
                await asyncio.sleep(2)

                game.set_state("game")
                await game.turn_next()



'''加载子模块'''
from . import first
from . import simple
from . import cert
from . import accomplice
from . import watch
from . import detect
from . import criminal
from . import police
from . import dog
from . import rumor
from . import exchange
from . import round_give

from . import info



'''打牌超时'''
import asyncio
from random import choice
from typing import TYPE_CHECKING
from .config import config, R

if TYPE_CHECKING:
    from .model import Player


def set_overtime_task(player: "Player"):
    '''设置超时任务'''
    loop = asyncio.get_event_loop()
    player.fut = loop.create_future()
    loop.create_task(overtime(player))


async def overtime(player: "Player"):
    try:
        await asyncio.wait_for(player.fut, config.overtime)
    except asyncio.exceptions.TimeoutError:
        game = player.game
        async with game.lock:
            card = choice(player.cards)
            player.cards.remove(card)
            await game.send(f"{player.index_name} 被系统强制丢弃了{card}")
            if card == R.犯人:
                player.is_good = False
                await game.end(True)
            else:
                await game.turn_next()



'''房间'''
from ayaka import AyakaChannel
from .cards import get_cards
from .shuffle import shuffle
from .overtime import set_overtime_task
from ..cat import cat
from ..model import Room, Game
from ..config import R


@cat.on_cmd(cmds=f"{R.犯人}在跳舞")
async def wakeup():
    '''唤醒猫猫'''
    await cat.start("room")
    await cat.send_help()
    room = cat.get_data(Room)
    room.users = []
    await join_room()


@cat.on_cmd(cmds="加入", states="room")
async def join_room():
    '''加入房间'''
    name = cat.user.name
    room = cat.get_data(Room)
    channel = AyakaChannel(
        type="private",
        id=cat.user.id
    )

    try:
        await cat.base_send(channel, "测试私聊，无需回复")
    except:
        await cat.send(f"{name} 不是bot好友，无法加入游戏")
    else:
        cat.add_listener(channel)

        room.join(cat.user.id, name)
        room.cards = []

        await cat.send(f"{name} 加入房间")
        await cat.send(room.info)


@cat.on_cmd(cmds="离开", states="room")
async def leave_room():
    '''离开房间'''
    name = cat.user.name
    room = cat.get_data(Room)
    channel = AyakaChannel(
        type="private",
        id=cat.user.id
    )

    room.leave(cat.user.id)
    room.cards = []

    cat.remove_listener(channel)

    await cat.send(f"{name} 离开房间")
    await cat.send(room.info)

    if not room.users:
        await cat.send("房间为空，游戏结束")
        await cat.rest()


@cat.on_cmd(cmds="房间", states="room")
async def show_room():
    '''查看房间信息'''
    # 排除私聊发送的消息
    if cat.event.origin_channel:
        return

    room = cat.get_data(Room)
    await cat.send(room.info)


@cat.on_cmd(cmds="开始", states="room")
async def start_game():
    '''开始游戏'''
    # 排除私聊发送的消息
    if cat.event.origin_channel:
        return

    room = cat.get_data(Room)

    # 人数控制
    n = len(room.users)
    if n < 3 or n > 8:
        return await cat.send(f"{R.犯人}在跳舞只能3-8人游玩，现在房间里有{n}人")

    # 生成牌库
    if not room.cards:
        room.cards = get_cards(n)

    # 洗牌
    shuffle(room.cards)

    # 状态转移
    cat.state = "game"

    # 初始化游戏
    game = cat.get_data(Game)
    game.init(room, cat.channel.id)

    # 通知
    items = ["游戏开始\n"]
    items.extend(p.index_name for p in game.players)
    await game.send("\n".join(items))

    # 发牌
    for player in game.players:
        items = ["您的手牌是\n", *player.cards]
        await player.send("\n".join(items))

    # 第一发现人
    await game.send(f"{R.第一发现人}是 {game.current_player.index_name}")

    # 设置超时任务
    set_overtime_task(game.current_player)



from random import randint


def _shuffle(cards: list[str]):
    n = len(cards)
    for i in range(n):
        j = randint(i, n-1)
        cards[i], cards[j] = cards[j], cards[i]
    return cards


def shuffle(cards: list[str]):
    for i in range(3):
        _shuffle(cards)
    return cards



import asyncio
from typing import Awaitable, Callable, Optional
from pydantic import BaseModel
from ayaka import AyakaChannel
from .cat import cat, help_dict
from .config import R, config
from .overtime import set_overtime_task


class User(BaseModel):
    '''房间成员'''
    id: str
    name: str


class Room(BaseModel):
    '''房间'''
    users: list[User] = []
    cards: list[str] = []

    @property
    def info(self):
        items = ["当前房间人员\n"]
        items.extend(u.name for u in self.users)
        return "\n".join(items)

    def join(self, id: str, name: str):
        for u in self.users:
            if u.id == id:
                return
        else:
            user = User(id=id, name=name)
            self.users.append(user)
            return True

    def leave(self, id: str):
        for u in self.users:
            if u.id == id:
                self.users.remove(u)
                return True


class Player(BaseModel):
    '''玩家'''
    index: int

    id: str
    name: str

    cards: list[str] = []
    '''当前手牌'''

    is_good: bool = True
    '''天生都是好人，打出共犯或犯人后变坏'''

    game: "Game"

    fut: Optional[asyncio.Future]
    '''超时控制'''

    choose_card: Optional[Callable[[str], Awaitable]]
    '''需要执行的任务（放一张卡牌）'''

    class Config:
        arbitrary_types_allowed = True

    @property
    def index_name(self):
        '''座号+名字'''
        return f"[座号{self.index+1}] {self.name}"

    async def send(self, msg: str):
        '''发送私聊消息'''
        await cat.base_send(AyakaChannel(type="private", id=self.id), msg)

    async def check(self, card: str, max_num: int = 4, at_require: bool = False):
        '''检查牌是否可以打出'''
        if card != R.第一发现人 and not self.game.first:
            await self.game.send("第一张牌必须是第一发现人")
            return False
        if self.game.current_player.id != self.id:
            await self.game.send("没轮到您")
            return False
        if card not in self.cards:
            await self.game.send(f"{self.index_name} 没有{card}")
            return False
        if len(self.cards) > max_num:
            await self.game.send(f"{card}只能在手牌<={max_num}时打出")
            return False
        if at_require and not self.game.get_player(cat.event.at):
            await self.game.send("您需要at一个游戏中的玩家")
            return False
        return True

    async def play_card(self, card: str):
        '''打出一张牌并通知'''
        self.cards.remove(card)
        await self.game.send(f"{self.index_name} 打出{card}")
        if config.auto_card_help:
            await self.game.send(help_dict[card])

        # 结束超时任务
        if self.fut and not self.fut.done():
            self.fut.set_result(True)


class MarkItem(BaseModel):
    '''标志物'''
    target_id: str = ""
    '''目标id'''
    owner_id: str = ""
    '''主人id'''


class GiveAction(BaseModel):
    giver: Optional[Player]
    receiver: Optional[Player]
    card: Optional[str]

    @property
    def ready(self):
        return self.giver and self.receiver and self.card

    def convey(self):
        if self.ready:
            self.giver.cards.remove(self.card)
            self.receiver.cards.append(self.card)


class RoundGive(BaseModel):
    gives: list[GiveAction] = []

    def init(self, givers: list[Player]):
        self.gives = [GiveAction(giver=p) for p in givers]

    def get_give(self, giver_id: str):
        for g in self.gives:
            if g.giver.id == giver_id:
                return g

    @property
    def all_given(self):
        '''所有人给牌完毕'''
        for g in self.gives:
            if not g.card:
                return False
        return True

    def set_receivers(self):
        '''手牌给上家'''
        gs = [self.gives[-1], *self.gives[:-1]]
        rs = [g.giver for g in gs]
        for r, g in zip(rs, self.gives):
            g.receiver = r

    async def convey_all(self):
        for g in self.gives:
            g.convey()
            await g.giver.send(f"{g.receiver.index_name} 获得了您的{g.card}")

        for g in self.gives:
            await g.receiver.send(f"您获得了 {g.giver.index_name} 的{g.card}")
            items = ["您当前的手牌\n", *g.receiver.cards]
            await g.receiver.send("\n".join(items))


class Game(BaseModel):
    '''游戏'''
    start: bool = False

    players: list[Player] = []
    '''玩家列表'''
    index: int = 0
    '''玩家指针'''
    first: bool = False
    '''第一发现人是否已打出'''

    detect_num: int = 0
    '''剩余侦探数量'''
    cert_num: int = 0
    '''剩余不在场证明数量'''

    dog: MarkItem = MarkItem()
    '''神犬标志物'''
    police: MarkItem = MarkItem()
    '''警部标志物'''

    lock: asyncio.Lock = asyncio.Lock()
    '''同步锁，保证用户操作的原子性，防止一个用户因某种情况连出两牌的情况'''
    group_id: str = ""
    '''群聊号'''

    round_give: RoundGive = RoundGive()
    '''环绕给牌'''

    class Config:
        arbitrary_types_allowed = True

    def init(self, room: Room, group_id: str):
        '''将房间成员转换为游戏玩家'''
        # 保存群号
        self.group_id = group_id

        # 将room成员转换为玩家
        self.players = [
            Player(
                id=u.id, name=u.name,
                index=i, game=self,
                cards=room.cards[i*4:(i+1)*4],
            )
            for i, u in enumerate(room.users)
        ]

        # 统计牌数
        self.detect_num = 0
        self.cert_num = 0
        for card in room.cards:
            if card == R.侦探:
                self.detect_num += 1
            elif card == R.不在场证明:
                self.cert_num += 1

        # 设置玩家指针
        for i, p in enumerate(self.players):
            if R.第一发现人 in p.cards:
                self.index = i
                break

        # 第一发现人未打出
        self.first = False

        # 正式开始
        self.start = True

    async def send(self, msg: str):
        '''发送群聊消息'''
        await cat.base_send(AyakaChannel(type="group", id=self.group_id), msg)

    @property
    def current_player(self):
        return self.players[self.index]

    def set_state(self, state: str):
        '''在群聊、私聊均可用'''
        cat.set_state(state, AyakaChannel(type="group", id=self.group_id))

    async def turn_next(self):
        '''转移至下一个有牌的玩家'''
        n = len(self.players)
        for i in range(n):
            self.index = (self.index+1) % n
            if self.current_player.cards:
                break

        await cat.send(f"现在轮到 {self.current_player.index_name} 出牌")
        set_overtime_task(self.current_player)

    def get_player(self, uid: str):
        for p in self.players:
            if p.id == uid:
                return p

    async def end(self, good_win: bool):
        '''游戏终结，给出输赢'''
        goods = []
        bads = []
        for p in self.players:
            if p.is_good:
                goods.append(p.name)
            else:
                bads.append(p.name)

        if good_win:
            winners, losers = goods, bads
        else:
            winners, losers = bads, goods

        items = [
            "，".join(winners) + "赢了",
            "，".join(losers) + "输了",
        ]
        await self.send("\n".join(items))
        await self.send("已返回房间")
        self.set_state("room")
        
        self.start = False


Player.update_forward_refs()



'''侦探'''
from ..cat import cat
from ..model import Game
from ..config import R


@cat.on_cmd(cmds=R.侦探, states="game", auto_help=False)
async def detect():
    # 排除私聊发送的消息
    if cat.event.origin_channel:
        return

    game = cat.get_data(Game)

    # 排除未参加游戏的人
    player = game.get_player(cat.user.id)
    if not player:
        return

    async with game.lock:
        card = cat.cmd
        if not await player.check(card, max_num=2, at_require=True):
            return

        game.detect_num -= 1
        await player.play_card(card)
        
        p2 = game.get_player(cat.event.at)
        await game.send(f"目标是 {p2.index_name}！")

        # 侦探排查中...
        if R.犯人 in p2.cards and R.不在场证明 not in p2.cards:
            player.is_good = True
            return await game.end(True)

        await game.send(f"{p2.index_name} 不是{R.犯人}~")
        await game.turn_next()



'''游戏局势'''
from ..cat import cat
from ..model import Game
from ..config import R


@cat.on_cmd(cmds="局势", states="*")
async def show_game_info():
    '''查看场面局势'''
    # 排除私聊发送的消息
    if cat.event.origin_channel:
        return

    game = cat.get_data(Game)

    if not game.start:
        return

    items = ["所有玩家剩余手牌数"]
    for p in game.players:
        items.append(f"{p.index_name} {len(p.cards)}张")

    if game.detect_num:
        items.append(f"剩余{R.侦探}牌 {game.detect_num}张")

    if game.cert_num:
        items.append(f"剩余{R.不在场证明}牌 {game.cert_num}张")

    items.append(f"目前轮到 {game.current_player.index_name}")

    await cat.send("\n".join(items))


@cat.on_cmd(cmds="手牌", states="*")
async def show_cards():
    '''私聊展示手牌'''
    # 只接受私聊发送的消息
    if not cat.event.origin_channel:
        return

    game = cat.get_data(Game)

    if not game.start:
        return

    player = game.get_player(cat.event.origin_channel.id)
    items = ["您的手牌\n", *player.cards]
    await player.send("\n".join(items))



'''交易'''
from asyncio import sleep
from .overtime import set_overtime_task
from ...cat import cat
from ...model import Game
from ...config import R


@cat.on_cmd(cmds=R.交易, states="game", auto_help=False)
async def exchange():
    # 排除私聊发送的消息
    if cat.event.origin_channel:
        return

    game = cat.get_data(Game)

    # 排除未参加游戏的人
    player = game.get_player(cat.user.id)
    if not player:
        return

    async with game.lock:
        card = cat.cmd
        if not await player.check(card, at_require=True):
            return

        if cat.event.at == player.id:
            return await game.send(f"不能和自己{R.交易}")

        p2 = game.get_player(cat.event.at)

        if not p2.cards:
            return await cat.send(f"{p2.index_name} 没有可{R.交易}的手牌")

        await player.play_card(card)
        await game.send(f"目标是 {p2.index_name}！")

        if not player.cards:
            await cat.send(f"{player.index_name} 没有可{R.交易}的手牌，此牌仍打出，但无效果")
            return await game.turn_next()

        # 记录两人
        game.round_give.init([player, p2])
        await game.send(f"{R.交易}进行中...\n请参与者通过私聊告知bot要给出的手牌")

        # 私聊通知
        for g in game.round_give.gives:
            await g.giver.send("请在这里发送您要给出的牌")
            # 设置超时任务
            set_overtime_task(g.giver)

        game.set_state("exchange")


@cat.on_cmd(cmds=[R.共犯, R.普通人, R.不在场证明, R.目击者, R.侦探, R.交易, R.谣言, R.情报交换, R.警部, R.犯人, R.神犬], states="exchange", auto_help=False)
async def set_exchange_card():
    # 只接受私聊发送的消息
    if not cat.event.origin_channel:
        return

    game = cat.get_data(Game)

    # 排除不参与交易的
    give = game.round_give.get_give(cat.user.id)
    if not give:
        return

    async with game.lock:
        card = cat.cmd
        if card not in give.giver.cards:
            items = ["你没有这张牌，你当前的手牌是\n", *give.giver.cards]
            return await give.giver.send("\n".join(items))

        give.card = card
        give.giver.fut.set_result(True)
        await game.send(f"{give.giver.index_name} 已决定好卡牌")

        # 判断是否完成
        if game.round_give.all_given:
            # 私聊 互相给牌
            game.round_give.set_receivers()
            await game.round_give.convey_all()
            await sleep(2)

            game.set_state("game")
            await game.turn_next()



'''第一发现人超时'''
import asyncio
from ..model import Player
from ..config import config, R


def set_overtime_task(player: Player):
    '''设置超时任务'''
    loop = asyncio.get_event_loop()
    player.fut = loop.create_future()
    loop.create_task(overtime(player))


async def overtime(player: Player):
    try:
        await asyncio.wait_for(player.fut, config.overtime)
    except asyncio.exceptions.TimeoutError:
        card = R.第一发现人
        player.cards.remove(card)
        player.game.first = True
        await player.game.send(f"{player.index_name} 被系统强制丢弃了{card}")
        await player.game.turn_next()



'''共犯'''
from ..cat import cat
from ..model import Game
from ..config import R


@cat.on_cmd(cmds=R.共犯, states="game", auto_help=False)
async def accomplice():
    # 排除私聊发送的消息
    if cat.event.origin_channel:
        return

    game = cat.get_data(Game)

    # 排除未参加游戏的人
    player = game.get_player(cat.user.id)
    if not player:
        return

    async with game.lock:
        card = cat.cmd
        if not await player.check(card):
            return

        player.is_good = False
        await player.play_card(card)
        await game.turn_next()



'''警部'''
from ..cat import cat
from ..model import Game
from ..config import R


@cat.on_cmd(cmds=R.警部, states="game", auto_help=False)
async def police():
    # 排除私聊发送的消息
    if cat.event.origin_channel:
        return

    game = cat.get_data(Game)

    # 排除未参加游戏的人
    player = game.get_player(cat.user.id)
    if not player:
        return

    async with game.lock:
        card = cat.cmd
        if not await player.check(card, max_num=2, at_require=True):
            return

        await player.play_card(card)

        game.police.target_id = cat.event.at
        game.police.owner_id = cat.user.id
        p2 = game.get_player(cat.event.at)

        await game.send(f"目标是 {p2.index_name}！")
        await game.turn_next()



'''猫猫'''
from ayaka import AyakaCat
from .config import R, config

cat = AyakaCat(f"{R.犯人}在跳舞")
cat.help = f'''
3-8人游玩，游玩前请先加bot好友，否则无法发牌。{config.overtime}s内不出牌会被系统强制弃牌（防止挂机）
'''

help_dict = {
    R.第一发现人: "一切都是由您开始。打出时没有特别效果",
    R.共犯: f"打出这张牌成为{R.共犯}。当{R.犯人}获胜时，您也获得胜利。当{R.犯人}输掉游戏时，您也跟着输掉游戏",
    R.犯人: f"您是{R.犯人}，不能让其他玩家知道。您只能在只剩下这张手牌时才能打出这张牌，如果您要这么做，您获胜",
    R.不在场证明: f"只要有了这个，您就不是{R.犯人}了。打出时没有特别效果。如果您有{R.犯人}和{R.不在场证明}，{R.侦探}质疑时您可以否认",
    R.侦探: f"您的手牌<=2才能打出这张牌。打出时质疑另一位玩家，如果该玩家持有{R.犯人}，您获胜",
    R.普通人: "打出时没有特别效果",
    R.谣言: "所有玩家随机抽一张他下家玩家的手牌",
    R.情报交换: "所有玩家把一张牌传给他上家玩家",
    R.目击者: "看另一个玩家的手牌",
    R.交易: "和至少还有1张手牌的玩家交换一张手牌。如果这是您打出的最后一张牌，则没有特别效果",
    R.神犬: f"选择一个玩家。该玩家弃掉他其中的一张手牌，并且获得{R.神犬}。如果他弃掉一张{R.犯人}牌，您获胜",
    R.警部: f"手牌<=2时，选定一个玩家放置此牌，若其最终打出{R.犯人}牌，您获得游戏胜利"
}


all_cards_str = f'''
所有手牌种类：{R.共犯}， {R.普通人}，{R.不在场证明}， {R.目击者}， {R.侦探}， {R.交易}， {R.谣言}， {R.情报交换}， {R.警部}， {R.犯人}，{R.神犬}，{R.第一发现人}
'''.strip()


@cat.on_cmd(cmds="卡牌帮助", states="*")
async def _():
    '''<卡牌名> 获取相应的帮助'''
    help = help_dict.get(cat.arg)
    if not help:
        await cat.send(all_cards_str)
        return await cat.send_many([f"[{k}] {v}" for k, v in help_dict.items()])
    await cat.send(help)

total_helps = f'''
开局每人4张手牌，轮流出牌
拥有{R.第一发现人}的人优先出牌（类似扑克规则的红桃3），且第一张牌必须是{R.第一发现人}

{R.犯人}牌只有在手牌数为1的时候才能打出，此时打出者作为{R.犯人}而胜利
其他人的目标就是在{R.犯人}逃跑成功之前，通过{R.侦探}、{R.神犬}、{R.警部}等牌抓到{R.犯人}，此时好人阵营胜利

当你打出{R.共犯}牌后，你便加入了坏人阵营，需要协助{R.犯人}获胜
当然，你也可以当个二五仔，若{R.共犯}使用{R.侦探}等牌抓到了{R.犯人}，那么他也视为好人阵营一同胜利

根据参与人数的不同，牌库的牌也不同，具体规则请发送 牌库规则 进一步了解

此外，当游戏进行中时
可以在群聊发送 局势，获得游戏进行情况等信息
或在私聊bot发送 手牌，获得你当前的手牌情况
'''.strip().split("\n\n")


@cat.on_cmd(cmds="详细帮助", states="*")
async def _():
    await cat.send_many(total_helps)


cards_build_helps = f'''
3人局，必须有{R.第一发现人}、{R.犯人}、{R.侦探}、{R.不在场证明}，加其他任意8张牌
4人局，必须有{R.第一发现人}、{R.犯人}、{R.侦探}、{R.不在场证明}、{R.共犯}，加其他任意11张牌
5人局，必须有{R.第一发现人}、{R.犯人}、{R.侦探}、{R.不在场证明}*2、{R.共犯}，加其他任意14张牌
6人局，必须有{R.第一发现人}、{R.犯人}、{R.侦探}*2、{R.不在场证明}*2、{R.共犯}*2，加其他任意16张牌
7人局，必须有{R.第一发现人}、{R.犯人}、{R.侦探}*2、{R.不在场证明}*3、{R.共犯}*2，加其他任意19张牌
8人局，加全部
'''.strip().split("\n")


@cat.on_cmd(cmds="牌库规则", states="*")
async def _():
    await cat.send_many(cards_build_helps)



'''目击者'''
from asyncio import sleep
from ..cat import cat
from ..model import Game
from ..config import R


@cat.on_cmd(cmds=R.目击者, states="game", auto_help=False)
async def watch():
    # 排除私聊发送的消息
    if cat.event.origin_channel:
        return

    game = cat.get_data(Game)

    # 排除未参加游戏的人
    player = game.get_player(cat.user.id)
    if not player:
        return

    async with game.lock:
        card = cat.cmd
        if not await player.check(card, at_require=True):
            return

        await player.play_card(card)

        p2 = game.get_player(cat.event.at)
        items = [f"{p2.index_name} 的手牌是\n", *p2.cards]

        await player.send("\n".join(items))
        await game.send(f"{R.目击者}观察中...")
        await sleep(2)
        await game.turn_next()



'''配置'''
from pydantic import BaseModel
from ayaka import AyakaConfig


class Rename(BaseModel):
    第一发现人: str = "第一发现人"
    犯人: str = "犯人"
    神犬: str = "神犬"
    警部: str = "警部"
    共犯: str = "共犯"
    普通人: str = "普通人"
    不在场证明: str = "不在场证明"
    目击者: str = "目击者"
    侦探: str = "侦探"
    谣言: str = "谣言"
    交易: str = "交易"
    情报交换: str = "情报交换"


class Config(AyakaConfig):
    __config_name__ = "犯人在跳舞"
    overtime: int = 90
    auto_card_help: bool = True
    '''每打一张牌，就自动发送该牌的帮助'''
    rename: Rename = Rename()


config = Config()
'''配置'''

R = config.rename
'''config.rename的缩写'''



'''谣言'''
from asyncio import sleep
from random import choice
from ..cat import cat
from ..model import Game
from ..config import R


@cat.on_cmd(cmds=R.谣言, states="game", auto_help=False)
async def rumor():
    # 排除私聊发送的消息
    if cat.event.origin_channel:
        return

    game = cat.get_data(Game)

    # 排除未参加游戏的人
    player = game.get_player(cat.user.id)
    if not player:
        return

    async with game.lock:
        card = cat.cmd
        if not await player.check(card):
            return

        await player.play_card(card)

        # 所有有牌的人
        ps = [p for p in game.players if p.cards]
        game.round_give.init(ps)

        # 被随机抽一张牌
        for g in game.round_give.gives:
            g.card = choice(g.giver.cards)

        # 设置接收方为上家
        game.round_give.set_receivers()

        await cat.send(f"{R.谣言}抽取中...")

        await game.round_give.convey_all()
        await sleep(2)
        await game.turn_next()
